#! /usr/bin/env python3
#
# ff.py
# Thursday, 01/27/2022
# copyright 2005-2022 by Paul Sherman
# TK/python script to find files, open them w/ default application
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#

import sys, os, string, re, subprocess, time, glob, stat, base64
from tkinter import *
from tkinter.commondialog import Dialog

APP_ICON_BASE64 = """
I2RlZmluZSBfd2lkdGggMTYKI2RlZmluZSBfaGVpZ2h0IDE2CiNkZWZpbmUgX3hfaG90IDAKI2Rl
ZmluZSBfeV9ob3QgMApzdGF0aWMgdW5zaWduZWQgY2hhciBfYml0c1tdID0gewogICAweGY4LCAw
eDAwLCAweGZjLCAweDAxLCAweDA2LCAweDAzLCAweDAzLCAweDA2LCAweDAzLCAweDA2LCAweDAz
LCAweDA2LAogICAweDAzLCAweDA2LCAweDAzLCAweDA2LCAweDA2LCAweDAzLCAweGZjLCAweDBm
LCAweGY4LCAweDFlLCAweDAwLCAweDNlLAogICAweDAwLCAweDdjLCAweDAwLCAweGY4LCAweDAw
LCAweGYwLCAweDAwLCAweDYwIH07Cg=="""

RUN_BASE64 = """
iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAACXBIWXMAAA7DAAAOwwHHb6hkAAAA
GXRFWHRTb2Z0d2FyZQB3d3cuaW5rc2NhcGUub3Jnm+48GgAAAAp0RVh0VGl0bGUAR2VhcgBuf8QA
AAMESURBVDiNnZXPi5VVGMc/z3nfe7X70xmQCKKZRmNsoeGUUG0kURIqjGigf6C2LRKyhSKtDEQ3
rvQ/EF24kApmUZsyJofm2g3zxm0QykVRNo4pc99zvi3e95157+tcGTpweM6P53z4nnOe8xzYROm1
2dFrmO81zN+qc2gza1x5oNtivNvgQHEsTngzEi4SLpY7MuRfY2+nzdhjwZ06uxE3IpjrNd2Z76Gy
VOOpyOy9GIiB2PRWfysTAvulzkeRs++iwI0fauwtsqzYWWwyXRU3I4wYcOJOBE8aOFPqIwMBAZY8
THqDBJHAa7tX+GpDMMDPTZuPxUuRIAIipdvKHTMo3sBndgC3p1f0rKVTG5+xE0sug63BN6rZnBNE
xk9FKIBdb7EzMqaqCcHgharZ6VjpeUaZdSopNkgytQmQGAxMx0Lg+kPAx/RtvsnlCrxTAWIZlQxW
yRTFqV1wZl9IVGV6w4vnc+hgGJ6PXYydscsyRXl1BevQJ60HfGZImeKP79bcSZOOm6375jW7uBk3
iHnF4AJCpUlc4MttDziVBUI+Hsb+DSccfP0IMBV4yQKvupf/YnnPPT4wx1wxVCy9ySvly81LhK4U
Qyr1Z27PPWZnVvhjPSoCVo49E1tGgb3xsNhXyljbmVtosL3T4iJwUCVHYW+PAhPsiCicUdo+2Gly
/to4LSfHNxKzsiJw7SHs/7vmPlUh3gV29wmOBcfrYngNhgneryR8GwsWgJ0lYNo2kHR8uWbvLsuu
mrG6LB0OMBPKvsX14qbNt5mKEqa2As7xYkV26v88kFXpqGBxNcZ50X8kV/Sa9nkcOJyDI0bkigyY
2UvPrWi2yBnKFQILgV2hkGA8I2qWHIJBMCbKAofAPzbYH0yTIVMVYMlbqixhfdveCB7u5AKCtG+x
yfRIsA/8E4zfPAwCOvrMfU0NvCYD3PaWqTSueenpP+9rwpudDUjB+N0C1bLqodJpM9Yt/Qb9mjvX
r5v6ddOvdT4sznUbHOi2GH8sdFS5VedQ/pn22uzYzJr/AK2tXhiz2hWRAAAAAElFTkSuQmCC"""

QTFM_BASE64 = """
iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAMAAADzapwJAAAABGdBTUEAALGPC/xhBQAAACBjSFJN
AAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAArlBMVEUAAACKykCNxkCMxj+K
xz2Kx0GMxj+LxT+MzECMxj+Mxj+OxjmNxz+Lxj+Mxj+Mxj+Nxj+Mxj+Mxj+Mxj+Mxj+LxUGOxkCM
xj+OxEGMyEGMxj+KxUCNxz6Nxj6Mxj+NyTyLyD2Mxj+Mxj+Nyj6KxUKMxkCLxUCLxj6Mxj+Mxj+M
xj+KyD6Lxj6Mxj+Lxj6Mxj+IzESGyUOMxkCMxj+Mxj+Lx0GMxj+Mxj+Mxj////+kxyrvAAAAOHRS
TlMAGJzpMjvnlhTg2hKR6t/+9fGvqvZPSN0rM+gwfzrMJi7FmR0jlFhj97+7JcD5i9MPE5De7jfc
iiwJ83cAAAABYktHRDnXAJVAAAAACXBIWXMAAA3XAAAN1wFCKJt4AAAAB3RJTUUH4gcZEQIlbL+r
rgAAAJlJREFUGNOV0NkOgkAMheEDKiCu4DK4gvuuuPf9n8wyJDqGwcT/ok2+myYFDLNAsmIp3Zbt
AEaZ6JuJ3ApMyjJVUdNxHQ0dN5FMz2/5nsptyR0AXRH0BNf/8ABKQz2PfvGYC1WOGCbIacrNVJgz
LP48mfBS5dWb15sttwuSuT9IznmVpWMBW8dHOG6WTzFwvlxvsuie7sczxguteD6iuwi88wAAACV0
RVh0ZGF0ZTpjcmVhdGUAMjAxOC0wNy0yNVQxNzowMjozNyswMjowMCLJeaAAAAAldEVYdGRhdGU6
bW9kaWZ5ADIwMTgtMDctMjVUMTc6MDI6MzcrMDI6MDBTlMEcAAAAAElFTkSuQmCC"""

ROX_BASE64 = """
R0lGODlhGAAYAMZWAFs8Hlw9Hl8/H2FAIGpFI3lPKHtQKX5SKYBUK4VXLIhZLY5dL49eL5NgMZRh
MZZiMZdjMplkM5xmM51nNKNrNqduN6lvOKtwOaxxOa5yOa9zOrB0O7J1O7d4Pbl6Pb18P8OHTcSI
T8eNV8eOWMiQW8iRXMmSXsqUYcuVY8yXZcyYZ86bbM+db8+ecNCgc9ChdNGidtGjeNWqgtarhNas
hdeuiNiwi9myjtu1ktu2lNy5l926mN27mt68nN69nd++n9/AoeHDpuHEqOLFqeLGq+PHrOPIruTJ
r+TKsOXLsubMtObNtefOt+fPuOfQuejRu+jSvOnTvurUv+rVwevWwuzYxf//////////////////
////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////yH5
BAEAAH8ALAAAAAAYABgAAAf7gH+Cg4SFhoeIiYqLihoZj5CMhBU1NzY1NDIwHJKCFDUioaErKRuQ
p4cTLiCsrScqKCgmJSQjIRmGESMfvB8eHBjBwsOchA0dGskaGzhQzs/PSBiFGBXWFhkzSj/c3d1A
F5M7TeTkRj086erqQxSEEj058vP09TlFE4QPQTr9/v8AdSyBQGiBEh8IEypc6MMJA0IKogCZSLGi
RSBNFBBCMGWIx48gQw5ZkoCQASlHUqpcyfJIkgMmkZRzQvOJzSc0nZQLYoBQgRhElDBpAiUKlSpV
qESB0mQJEiEvChAa8KCFi6tYs2Zl4UBAIQAEwoodS5ZAgE5o06qVFAgAOw=="""

INFO_BASE64="""
R0lGODlhGAAYAOeGAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAADJopDNmpS5clSxYjyZPfSZOeylThidPgCZNfCZMeyVJdyRJ
djVmpDZnpTRloURyq0RyrEVyrEh0rUh1rURxq0RxrEVxrFmHvVmIvlqIvlqJvluJv1uKv1yKv1yK
wFyLwF2LwF6MwV6NwV+NwV+Nwl+OwmCOwmCOw2GPw2KQxGORxGORxWSRxWSSxWWSxmWTxmaUxmaU
x2eUx2eVx2iVyGiWyGmWyGmXyWqXyWqYyWuYymuZymyZy22ay22azG2bzG6bzG+czXCdzoClz4Oo
0Ieq0out1JGz16TA3q7G4rDH4LHH4LbK4rbL4rbL47fM47fM5LfN5LjN5LjN5bjO5bjO5rnP5rrP
57vQ58bX68/d7dDc69Dd69Le7NXh7trk8N3m8d3n8d3n8t7n8d7n8t/o8t/o8+Lr9OPr9Ozy+PH1
+vP2+vf5/Pj6/fn7/f7+/v//////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////yH5BAEKAP8ALAAAAAAYABgA
AAj+AP8JHEiwoMGDCBMqJDhixEKFKOrQOfHwYIo+Z8zwoVhxYAo/apAUMbOHY8WPbJ4sQTKEpEmF
H91UkeJESZEgLhfGxHJlCpQlRoT4KKPnZcGPba7wFEQIEJwtPnSQ0WPiIIo+a6JQsWLljZcvhvD4
yFFDTJ4SBUfUSbPESRQpUZ4w4WLIT48bM1yIiSOCIZ00SJY0Gbyky6BAWXTYiNEizJy+BE/wSWME
SZIkR/4Y0sLjBg0YYOygNSgZzZAiRIYA+dEjx+Iwd0YfPLEHzY/bhgzhsCEjzNmFtMvw2JHjBm/f
VR+y2FPGho0ZMXyv6PiPhR4yMF6AuaOCusAVecYUhLHT3ft3OXPKmxdIgsT69/AFBgQAOw=="""

COPY_BASE64="""
iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
B3RJTUUH1QoOAB42P7Pi6AAAAqRJREFUOMullU1IVFEYhp9z586U5TTqOP4kQiAouQtz4SYsckRa
ZNYmsJ27yG3TIhhqkDbZIgrCCYIEt7XRXbkoCdKFBEJBIGbozJjOvXMdm3E8p8X864yDeeFyvvPz
Pbz3/Q73E1R4hoeHPwK9+5ZnJycnLx+WJwonfj/a8iwOgGTj9Qv6iTPvpVSerq4OhEgfVUqxsPAd
TRORVCI25Ai9mwc410vS70ceAI/22e4o5HMpqQb4W9ujNbb3iNbWxpKKVlcjrP+YUyc356QCNIGl
pHbvxYe9twB6VunWZ/ls6FKbK7TbwFezkx1zD7fbQ1tbe05DWnR6TCRgfb1DVLWct3WfWcJFyDU9
93McyIOXZ3E4dWrsdo35zU58D59gRA3Gx5+yuLhUUrHTWY3P94DaulrGHt1nqGkDqUQ9KHLgnC9K
YCXAXedGt9kIBAKkUqmSYF3XEULgctUQT4riYu0HH6isENjt9rL7SqmCSQVw4f7MzMwhUOjv95bd
P1TxwMBASZXpVyKlPAJYlVechypA0dfXdzzFeYXpMQs3TYOGhqbK4H3+Mz09A6iiz/d6vbnYjJll
c/VyPkgp6e/3FimUMu1rdi0SjhzdikJAoRVra7+xrG3i8ThKyeOCZUaxYmXlF4ODN3PnDCN6VPBe
ETRnRUalYUQxTRPLiv2P4nyRti2LcDiMytxd0zQJhdaZeD3BtaaF8uBs2U47JJGNCM3NLblDLfuS
DCNaBHU7tisrbpXfGHvsI57UKjUXbrV84ZQtWd6KN7MkRq+yae2kPBfPGnSrTxx2v0v1oaiVAlT4
QAe5e4UbQtNeopSnRNsS5f5F+UBsyarGwG5T76tgcGrnQMLIyO0qoAaoA+oBD9CQiZ1AHNgCNoAI
8AfYzIyxYHBKAvwDXmGHizkmUGsAAAAASUVORK5CYII="""

CANCEL_BASE64 = """
R0lGODlhFgAWAOf6AFYAAF4AAG4AAHQAAIIAAIMAAIQAAIYAAIgAAIkAAIkEBIgFBYkHB4wHB4kI
CIoJCbUAALkAALoAALsAAL0AAL4AAL8AAMAAAMEAAMIAAMMAAMUAAMYAAMgAAJkNDbMGBrsEBMsA
ALUGBr0EBL8EBMAFBZsPD8IFBcMFBZwQEKIPD6gPD50SEp8UFKwREcsLC88QEL8WFssWFrocHLod
HcIdHcMfH8UgINEdHcYgINMeHqkqKqwqKtUfH6krK6srK7coKMgkJNggILspKawuLsgnJ8cpKa8y
Mr4uLsosLL8wMK81NdkqKrY1Nc0xMbA5OcszM7A7O8w0NM41Nc02NrE+Ps83N884ONA4ONE5Ocs7
O8g8PNE6OtE7O9I7O9I8PNQ8PNI9PbtERL1ERMFDQ8VCQtg9PdE/P8VDQ9g+PsBFRcRERNc/P9g/
P71HR8JGRsFHR9lBQcxFRdlCQs1GRtlDQ8VJSdlERMxISNlFRcRLS81JSdpGRtpHR+RFRdpISM5M
TNpJScBRUdpKSs1OTsxPT9tLS9pMTMFTU9tMTMBUVMxRUdtNTdRPT8FVVcJVVc5SUulLS9tPT91P
T8BXV9pQUMVWVsJXV9xQUOpNTdxRUdBVVdxSUttTU99SUslZWeRTU8NcXOpSUuRUVOFVVctbW8Vd
XeFXV8RfX8VfX+lWVutWVuZYWM5fX8ViYsZiYsVjY8ZjY+laWudbW8dkZOhdXchmZsZnZ+1dXehf
X+NhYeViYupiYuNkZORoaO1mZu9mZuVqau9paexqau1sbO9sbMx2duxvb+pycsx7e8t8fO92dvF7
e8+IiPKBgfKIiPSJifWMjNWVlfWNjfWOjvSPj/WPj/aQkPaSkvSTk/SUlPSVlfaWlveYmPeZmdmi
oveamvebm+S8vN3T097U1N/V1eXV1eXW1ubW1uLY2ObY2ObZ2ejf3+bi4uri4uvj4+nk5O7j4/Hj
4/Dk5PXm5uvr6+3s7Pbq6u7u7vDu7v///////////////////////yH5BAEKAP8ALAAAAAAWABYA
AAj+AP8JHEiwoMGDCRAkWMhQ4UGCCapoCwfOWzdu2J7xSPDwH4Io2oJJYmQoEJ86p5z94GgQwZNt
wThpkpRoUJ87cUgxI8JyIIIl2nx1KrUJEyObdOyw8bTMBwKfR7L1qvRJGSxIR/coSvYGzKRjO57+
U1BNV6JGj8aNQ0UIkCN17txoyXJo2AOBDa79McRokSty5UJdWvfOVBkvWKYUc4DXGp9AgwwVQpWu
Hbx3qdB84WJFijAGeKnd4dPnTyA9zerhk0YmTJcrVKD8Av2vwbQ5dfLkwYPonL188V61cpKkiBFi
C/BGaxNnjhxB5twZizVPnhgkFihMyHQALzQzac6XOELH7tYaEbTu0WsCYUIMWd3HAmMC4wWcb5Ro
bNAww5a4MRFIUAMy8f3TAi5ChCCDGkrsl8EFH1iyhQQ2gDKEWAKxsEoPHXDg4AUVaCfBDaMAgeFA
KYiiw34YgChiDqO4cCJBJkSCAwonlEDCCCAEMcoKMxIkgAd+qMJLLrXMwsouKgzQkUAAGGBAAQUQ
QKUBATyp5UEBAQA7"""

def istext(path):
    return (re.search(r':.* text', subprocess.Popen(["file", '-L', path], stdout=subprocess.PIPE).stdout.read())
            is not None)

class Chooser(Dialog):
    command = "tk_chooseDirectory"
    def _fixresult(self, widget, result):
        if result:
            # keep directory until next time
            self.options["initialdir"] = result
        self.directory = result # compatibility
        return result

class StatusBar(Frame):
    def __init__(self, master):
        Frame.__init__(self, master)
        self.label = Label(self, bd=1, relief=SUNKEN, anchor=W, font=("Sans",12), bg="#F6F1DF")
        self.label.pack(expand=0, fill=X)

    def set(self, format, *args):
        self.label.config(text=format % args)
        self.label.update_idletasks()

    def clear(self):
        self.label.config(text="")
        self.label.update_idletasks()

class CantOpen(Toplevel):
    def __init__(self, parent, msg1, title = "Cannot Open"):
        Toplevel.__init__(self, parent)
        self.transient(parent)
        if title:
            self.title(title)
        self.parent = parent
        self.result = None
        body = Frame(self)
        self.initial_focus = self.body(body, msg1)
        body.pack(padx=16, pady=32)
        self.buttonbox()
        if not self.initial_focus:
            self.initial_focus = self
        self.protocol("WM_DELETE_WINDOW", self.ok)
        self.geometry("+%d+%d" % (parent.winfo_rootx()+20, parent.winfo_rooty()+140))
        self.config(bg="#cccccc")
        self.initial_focus.focus_set()
        self.update_idletasks()
        try:
            self.grab_set()
        except:
            pass
        self.wait_window(self)

    def body(self, master, m1):
        self.l11 = Label(master, name='l11',text=m1, font=("Sans", 16), bg="#cccccc")
        self.l11.pack()

    def buttonbox(self):
        box = Frame(self)
        box.config(bg="#cccccc")
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE,bg="#cccccc")
        w.pack(side=LEFT, padx=5, pady=5)
        self.bind("&lt;Return>", self.ok)
        box.pack()

    def ok(self, event=None):
        self.parent.focus_set()
        self.destroy()

class Dialog(Toplevel):
    def __init__(self, parent, msg1, msg2, msg6, msg7, title = None):
        Toplevel.__init__(self, parent)
        self.transient(parent)
        if title:
            self.title(title)
        self.parent = parent
        self.result = None
        body = Frame(self)
        self.initial_focus = self.body(body, msg1, msg2, msg6, msg7)
        body.pack(padx=5, pady=5)
        self.buttonbox()
        if not self.initial_focus:
            self.initial_focus = self
        self.protocol("WM_DELETE_WINDOW", self.ok)
        self.geometry("+%d+%d" % (parent.winfo_rootx()+10, parent.winfo_rooty()+150))
        self.config(bg="#cccccc")
        self.initial_focus.focus_set()
        self.update_idletasks()
        try:
            self.grab_set()
        except:
            pass
        self.wait_window(self)

    def body(self, master, m1, m2, m6, m7):

        self.leftsubframe = Frame(master, name='leftsubframe',bg="#3D80C3")
        self.leftsubframe.pack(side=LEFT, expand=1, fill=BOTH)
        self.rightsubframe = Frame(master, name='rightsubframe',bg="#3D80C3")
        self.rightsubframe.pack(side=RIGHT, expand=1, fill=BOTH)

        self.l11 = Label(self.leftsubframe, name='l11',text="file name", font=("Aerial", 12, "bold"), bg="#3D80C3", fg="#FFFFFF")
        self.l11.pack(side=TOP,pady=4, padx=12)

        self.l22 = Label(self.leftsubframe, name='l22',text="file size", font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l22.pack(side=TOP,pady=4, padx=12)

        self.l66 = Label(self.leftsubframe, name='l66',text="permissions", font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l66.pack(side=TOP,pady=4, padx=12)

        self.l77 = Label(self.leftsubframe, name='l77',text="owner", font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l77.pack(side=TOP,pady=4, padx=12)

        self.l1 = Label(self.rightsubframe, name='l1',text=m1, font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l1.pack(side=TOP,pady=4, padx=12)

        self.l2 = Label(self.rightsubframe, name='l2',text=m2, font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l2.pack(side=TOP,pady=4, padx=12)

        self.l6 = Label(self.rightsubframe, name='l6',text=m6, font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l6.pack(side=TOP,pady=4, padx=12)

        self.l7 = Label(self.rightsubframe, name='l7',text=m7, font=("Aerial", 12, "bold"),bg="#3D80C3", fg="#FFFFFF")
        self.l7.pack(side=TOP,pady=4, padx=12)

    def buttonbox(self):
        box = Frame(self)
        box.config(bg="#3D80C3")
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE, bg="#3D80C3", fg="#FFFFFF")
        w.pack(side=LEFT, padx=5, pady=5)
        self.bind("&lt;Return>", self.ok)
        box.pack()

    def ok(self, event=None):
        self.parent.focus_set()
        self.destroy()

class Finder:
	def __init__(self, master=None):
		self.search_cancelled = None
		self.foundflag = None
		self.count = 0
		self.isuser = (os.environ['USER'] != "root")
		if len(sys.argv) > 1:
			if os.path.isdir(sys.argv[1]):
				self.search_path = sys.argv[1]
			else:
				self.search_path = os.getenv('HOME')
		else:
			#self.search_path = os.getenv('HOME')
			self.search_path = os.getcwd()


		self.cancelled = False
		start_text = "Search results will be displayed here."
		self.frame = Frame(master, name="frame")
		self.frame.winfo_toplevel().title('File-Find 4.0')
		self.frame.pack(expand=1, fill=BOTH)
		self.master = self.frame.master
		self.subframe = Frame(self.frame, name="subframe")
		self.subframe.pack(expand=0, fill=BOTH)
		self.leftsubframe = Frame(self.subframe, name='leftsubframe', bg="#aaaaaa")
		self.leftsubframe.pack(side=LEFT, expand=1, fill=BOTH)
		self.rightsubframe = Frame(self.subframe, name='rightsubframe', bg="#aaaaaa")
		self.rightsubframe.pack(side=RIGHT, expand=1, fill=BOTH)
		self.optsubframe = Frame(self.leftsubframe, name='optsubframe', bg="#aaaaaa")
		self.optsubframe.pack(side=BOTTOM, expand=1, fill=BOTH)
		self.cmdsubframe = Frame(self.rightsubframe, name='cmdsubframe', bg="#aaaaaa")
		self.cmdsubframe.pack(side=BOTTOM, expand=1, fill=BOTH)

		self.chaptervar = StringVar(master)
		self.chapter = Button(self.rightsubframe, name='chapter', text='Choose Start Directory', font=("Aerial", 11, "bold"), command=self.askdirectory, bg="#999999", fg="#f9f9ff")
		self.chapter.pack(side=TOP, pady=8)

		self.eStart = Entry(self.rightsubframe, name='eStart', fg="#006600", borderwidth=1, width=20, font=("Aerial", 10, "bold"), bg="#FFFFFF")
		self.eStart.pack(expand=0, fill=X, padx=4, pady=4)
		self.eStart.insert('end', self.search_path)

		self.bsearch = Button(self.cmdsubframe, name='bsearch', text='Start Search', font=("Aerial", 11, "bold"), command=self.search, bg="#80aa80", fg="#ffffff")
		self.bsearch.pack(side=LEFT, pady=4, padx=12)

		self.bstop = Button(self.cmdsubframe, name='bstop', text='Stop Search', font=("Aerial", 11, "bold"), command=self.stop_search, bg="#996666", fg="#FFFFFF")
		self.bstop.pack(side=LEFT, pady=4, padx=12)

		self.bclose = Button(self.cmdsubframe, name='bclose', text='Close', font=("Aerial", 11, "bold"), command=self.close, bg="#000000", fg="#ffffff")
		self.bclose.pack(side=RIGHT, pady=4, padx=12)

		self.l1 = Label(self.leftsubframe, name='l1',text='Enter a name \nor partial name to find', font=("Aerial", 11, "bold"), bg="#aaaaaa", fg="#FFFFFF")
		self.l1.pack(expand=0, fill=X, padx=8)

		self.search_hidden = BooleanVar()
		self.ohidden = Checkbutton(self.optsubframe, name='ohidden',text='Include hidden (.) and /cache files', variable=self.search_hidden,relief=FLAT, bg="#aaaaaa")
		self.ohidden.pack(expand=0, fill=X, pady=4, padx=8)
		self.search_hidden.set(False)

		self.entry = Entry(self.leftsubframe, name='entry',relief=SUNKEN, font=("Aerial", 11, "bold"), borderwidth=2,width=18, bg="#FFFFAA")
		self.entry.pack(expand=0, fill=X, padx=8)

		self.casevar = BooleanVar()
		self.casesense = Checkbutton(self.optsubframe, name='casesense',text='Case insensitive', variable=self.casevar,relief=FLAT, bg="#aaaaaa")
		self.casesense.pack(side=LEFT, pady=4, padx=12)
		self.casevar.set(True)

		self.recursevar = BooleanVar()
		self.orecurse = Checkbutton(self.optsubframe, name='orecurse',text='Recursive', variable=self.recursevar,relief=FLAT, bg="#aaaaaa")
		self.orecurse.pack(side=RIGHT, pady=4, padx=12)
		self.recursevar.set(True)

		RUN_IMG = PhotoImage(data=RUN_BASE64)
		label0 = Label(image=RUN_IMG)
		label0.image = RUN_IMG

		INFO_IMG = PhotoImage(data=INFO_BASE64)
		label1 = Label(image=INFO_IMG)
		label1.image = INFO_IMG

		QTFM_IMG = PhotoImage(data=QTFM_BASE64)
		label2 = Label(image=QTFM_IMG)
		label2.image = QTFM_IMG

		ROX_IMG = PhotoImage(data=ROX_BASE64)
		label3 = Label(image=ROX_IMG)
		label3.image = ROX_IMG

		COPY_IMG = PhotoImage(data=COPY_BASE64)
		label4 = Label(image=COPY_IMG)
		label4.image = COPY_IMG

		CANCEL_IMG = PhotoImage(data=CANCEL_BASE64)
		label = Label(image=CANCEL_IMG)
		label.image = CANCEL_IMG


		self.frame.vscroll = Scrollbar(self.frame, orient=VERTICAL, bg="#cccccc")
		self.frame.list = Listbox(self.frame,relief=SUNKEN,font=("Courier", 11),width=40, height=10,selectbackground='#eed5b7',bg="#dfdfdf",selectborderwidth=0,selectmode=BROWSE,yscroll=self.frame.vscroll.set)
		self.frame.vscroll['command'] = self.frame.list.yview
		self.frame.vscroll.pack(side=RIGHT, fill=Y)
		self.frame.list.pack(expand=1, fill=BOTH)
		if os.environ['USER'] != "root":
			elf.frame.list.insert(0, "A result in LIGHT GRAY indicates that ")
			self.frame.list.insert(END, "you do not have write permissions for that file.")
			self.frame.list.insert(END, "(Although most can be opened from the application as READ-ONLY)")
		self.frame.list.insert(END, "")
		self.frame.list.insert(END, " Double-clicking a listed item will show the file properties.")
		self.frame.list.insert(END, "")
		self.frame.list.insert(END, " Right-clicking an item will open a menu of choices,")
		self.frame.list.insert(END, " Including opening the file with its default program")
		self.frame.list.insert(END, " opening its location with the File Manager")
		self.frame.list.insert(END, " or with Arox.")
		self.frame.list.insert(END, "")
		self.frame.list.insert(END, " Binary executable files are NOT allowed ")
		self.frame.list.insert(END, " to be opened from this application.")
		self.frame.list.insert(END, "")
		self.frame.list.insert(END, " Hope you find what you're looking for!")
		self.frame.list.insert(END, "  -- Paul")
		self.popup = Menu(self.frame, tearoff=0, font=('sans', 15, 'bold'), cursor=("top_left_arrow"), fg=("#888"))
		self.popup.add_command(label=" Open w/ Default Application", command=self.open_file, compound=("left"), image=RUN_IMG )
		self.popup.add_separator()
		self.popup.add_command(label=" Show in File Manager", command=self.do_file, compound=("left"), image=QTFM_IMG)
		self.popup.add_command(label=" Show in Arox", command=self.do_file_rox, compound=("left"), image=ROX_IMG)
		self.popup.add_command(label=" Copy Path", command=self.do_copy_path, compound=("left"), image=COPY_IMG)
		self.popup.add_separator()
		self.popup.add_command(label=" About FileFinder", command=self.show_about, compound=("left"), image=INFO_IMG)
		self.popup.add_command(label=" Cancel",compound=("left"), image=CANCEL_IMG )

		#self.status = StatusBar(self.frame)
		#self.status.pack(side=BOTTOM, fill=X)

		self.v = StringVar()
		self.status = Label(master, textvariable=self.v, bd=1, relief=SUNKEN, anchor=W)
		self.status.pack(side=BOTTOM, fill=X)

		self.frame.list.bind("<ButtonRelease-1>", self.do_popup)
		self.frame.list.bind("<Button-3>", self.show_info)

		self.bstop.config(state='disabled')
		#self.status.set(" Search results will be displayed in the above. LEFT CLICK to select item, RIGHT CLICK to view action menu.")
		self.v.set(" Search will be displayed above.")
		self.entry.focus_set()


	def close(self):
		self.frame.quit()


	def do_popup(self, event):
		try:
			self.popup.tk_popup(event.x_root, event.y_root, 0)
		finally:
			self.popup.grab_release()

	def show_about(self):
		CantOpen(self.frame, 'File finding utility for Linux systems.\nUtilizes Python, Tk(distributed with Python)\nand some system commands to speed things up a pinch.\n\nWritten by Paul Sherman\nhttp://www.absolutelinux.org\nLicense:GPL3', "Absolute Linux File Finder")

	def do_file(self):
		try:
			f = self.frame.list.get(self.frame.list.curselection()[0])
		except:
			return
		if os.path.isfile(f):
			os.system('qtfm %s&' % os.path.dirname(f))

	def do_file_rox(self):
		try:
			f = self.frame.list.get(self.frame.list.curselection()[0])
		except:
			return
		if os.path.isfile(f):
			os.system('/usr/libexec/arox/arox %s&' % os.path.dirname(f))


	def open_file(self):
		try:
			f = self.frame.list.get(self.frame.list.curselection()[0])
		except:
			return
		subprocess.Popen(["xdg-open", f])


	def do_copy_path(self):
		try:
			f = self.frame.list.get(self.frame.list.curselection()[0])
		except:
			return
		if os.path.isfile(f):
			os.system("echo -n {s} | xsel -bi".format(s=f))


	def sizeof_fmt(self, num):
		for x in ['bytes','KB','MB','GB']:
			if num < 1024.0 and num > -1024.0:
				return "%3.1f %s" % (num, x)
			num /= 1024.0
		return "%3.1f %s" % (num, 'TB')


	def show_info(self, e):
		try:
			f = self.frame.list.get(self.frame.list.curselection()[0])
		except:
			return
		if not os.path.isfile(f):
			return
		try:
			file_stats = os.stat(f)
		except:
			CantOpen(self.frame, "Could not stat the file:\n%s" % f, "Property Retrieval Error")
			return
		# create a dictionary to hold file info
		file_info = {
			'fname': f,
			'fsize': file_stats [stat.ST_SIZE],
			'f_mo': oct(file_stats[stat.ST_MODE] & 0o777)
		}

		msg1 = "%(fname)s" % file_info
		mTMP = "%(fsize)s" % file_info
		msg2 = self.sizeof_fmt(float(mTMP))
		msg6 = "%(f_mo)s" % file_info
		msg7 = "Unknown"
		retOwner = subprocess.getoutput('ls -l "%s"' % f)
		if retOwner:
			try:
				msg7 = string.split(retOwner, " ")[2]
			except:
				pass
		Dialog(self.frame, msg1, msg2, msg6, msg7, "File Information")


	def stop_search(self):
		self.cancelled = True
		self.bstop.config(state='disabled')
		self.bsearch.config(state='normal')

	def search(self, opt="null"):
		self.cancelled = False
		if self.entry.get() == "":
			return
		if not os.path.isdir(self.eStart.get()):
			CantOpen(self.frame, "Improper path enetered to start search.", "NOT A DIRECTORY")
			return
		# clear the listbox
		self.frame.list.delete('0', 'end')
		self.bsearch.config(state='disabled')
		self.bstop.config(state='normal')
		# set files found flag back to 0
		self.foundflag = None
		# search
		ret = self.Walk(self.eStart.get(), self.recursevar.get(), self.entry.get())
		if self.foundflag == None:
			self.v.set("No Files Found")
		else:
			self.v.set('%s Matching Files Found.       << LEFT CLICK to select item       >> RIGHT CLICK to view action menu.' % self.frame.list.size())
		self.bstop.config(state='disabled')
		self.bsearch.config(state='normal')

	def askdirectory(self, **options):
		tmpVar =  self.eStart.get()
		while self.eStart.get():
			self.eStart.delete(0)
		self.eStart.insert(0, Chooser(*(), options).show())
		# lastly we return what was there in case user cancelled looking for a new directory
		if self.eStart.get() == "":
			self.eStart.insert(0, tmpVar)


	def Walk(self, root, recurse=False, pattern='*'):
		if self.cancelled:
			return 0

		pattern = pattern or '*'
		# must have at least root folder

		# Get matches in top directory, find command dependent on case sesitivity
		if self.casevar.get():
			tstr = subprocess.getoutput('find "%s" -maxdepth 1 -iname "*%s*" 2>/dev/null' %  (root, pattern))
		else:
			tstr = subprocess.getoutput('find "%s" -maxdepth 1 -name "*%s*" 2>/dev/null' %  (root, pattern))
		if len(root) > 64:
			statline = root[-64:]
		else:
			statline = root
		self.v.set(statline)
		if tstr:
			tstrings = str.split(tstr,'\n')
			for astr in tstrings:
				if not os.path.isdir(astr):
					self.frame.list.insert(0,astr)
					if self.isuser:
						if not os.access(astr, os.W_OK):
							self.frame.list.itemconfigure(0, foreground="#999999")
					self.foundflag = True

			self.frame.list.update_idletasks()
		if self.recursevar.get():
			try:
				dirStr = subprocess.getoutput('find "%s" -type d -maxdepth 1' % root)
			except os.error:
				return 0
			names = str.split(dirStr, '\n')
			for name in names[1:]:
				if not self.search_hidden.get():
					if name.rfind('/cache') != -1 or name.rfind('/.') != -1:
						continue
				self.count += 1
				if self.count > 20:
					self.count = 0
					self.frame.list.update()
				if self.cancelled:
					break
				if name == "/cache":
					continue
				ret = self.Walk(name, recurse, pattern)
			return 1

	def stop_pressed(self):
		self.cancelled = True

def main():
    root = Tk()
    sb = Finder(root)
    w,h = (700, 500)
    root.minsize(w, h)
    screenwidth=root.winfo_screenwidth()
    screenheight=root.winfo_screenheight()
    x=(screenwidth-w)/2
    y=(screenheight-h)/2
    root.geometry("%dx%d%+d%+d" % (w, h, x, y))

    # To use existing X11 xbm bitmap
    #root.iconbitmap("@/usr/X11R6/include/X11/bitmaps/target")
    #
    # load application icon from remorte file
    # seems to be the only way to get a color app icon on Linux
    # also seems to have to e a GIF
    try:
        img = PhotoImage(file='/usr/share/pixmaps/fileFinder.gif')
        root.tk.call('wm', 'iconphoto', root._w, img)
    except:
        # if our nice color icon is not found...
        # we have to generate another from internal data, or the application will error out.
        # to specify icon, on Linux it has to be a XBM
        # has to be linked to file, not directly to data,
        # and full path must be preceeded with an "@"
        icondata= base64.b64decode(APP_ICON_BASE64)
        tempFile= "/tmp/icon.xbm"
        iconfile= open(tempFile,"wb")
        iconfile.write(icondata)
        iconfile.close()
        root.wm_iconbitmap("@/tmp/icon.xbm")
        os.remove(tempFile)
    root.bind('<Return>', sb.search)
    root.mainloop()
main()
